@@ -235,7 +235,7 @@ applet_callback_callback (GtkWidget      *widget,
 		if (!strcmp (menu->name, "launch"))
 			launcher_launch (menu->info->data, widget);
 		else if (!strcmp (menu->name, "properties"))
-			launcher_properties (menu->info->data, screen);
+			launcher_properties (menu->info->data);
 		break;
 	case PANEL_OBJECT_DRAWER: 
 		if (strcmp (menu->name, "add") == 0) {
@@ -537,7 +537,7 @@ panel_applet_create_menu (AppletInfo *info)
 		gtk_widget_show (menuitem);
 
 		menuitem = gtk_check_menu_item_new ();
-		setup_menuitem (menuitem, GTK_ICON_SIZE_MENU, NULL, _("_Lock To Panel"), FALSE);
+		setup_menuitem (menuitem, GTK_ICON_SIZE_MENU, NULL, _("Loc_k To Panel"), FALSE);
 		gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (menuitem),
 						locked);
 		g_signal_connect (menuitem, "toggled",
@@ -1223,7 +1223,7 @@ panel_applet_register (GtkWidget       *applet,
 		}
 
 		if (!l) {
-			g_warning (_("Can't find an empty spot"));
+			g_warning (_("Cannot find an empty spot"));
 			panel_profile_delete_object (info);
 			return NULL;
 		}
