@@ -30,6 +30,7 @@
 #include <gdk/gdkkeysyms.h>
 #include <libgnomevfs/gnome-vfs-result.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
+#include <libgnomevfs/gnome-vfs-utils.h>
 #include <libgnomevfs/gnome-vfs-xfer.h>
 #include <gconf/gconf-client.h>
 
@@ -1122,11 +1123,14 @@ drag_data_get_menu_cb (GtkWidget        *widget,
 		       guint             time,
 		       GMenuTreeEntry   *entry)
 {
-	char *uri_list;
+	const char *path;
+	char       *uri;
+	char       *uri_list;
 
-	uri_list = g_strconcat (gmenu_tree_entry_get_desktop_file_path (entry),
-				"\r\n",
-				NULL);
+	path = gmenu_tree_entry_get_desktop_file_path (entry);
+	uri = gnome_vfs_get_uri_from_local_path (path);
+	uri_list = g_strconcat (uri, "\r\n", NULL);
+	g_free (uri);
 
 	gtk_selection_data_set (selection_data,
 				selection_data->target, 8, (guchar *)uri_list,
@@ -1531,10 +1535,19 @@ create_menuitem (GtkWidget          *menu,
 				     menu_item_targets, 1,
 				     GDK_ACTION_COPY);
 
-		//FIXME should remove .png at the end of the icon name
-		if (gmenu_tree_entry_get_icon (entry) != NULL)
-			gtk_drag_source_set_icon_name (menuitem,
-						       gmenu_tree_entry_get_icon (entry));
+		if (gmenu_tree_entry_get_icon (entry) != NULL) {
+			const char *icon;
+			char       *icon_no_ext;
+
+			icon = gmenu_tree_entry_get_icon (entry);
+			if (!g_path_is_absolute (icon)) {
+				icon_no_ext = panel_util_icon_remove_extension (icon);
+				gtk_drag_source_set_icon_name (menuitem,
+							       icon_no_ext);
+				g_free (icon_no_ext);
+			}
+		}
+
 		g_signal_connect (G_OBJECT (menuitem), "drag_begin",
 				  G_CALLBACK (drag_begin_menu_cb), NULL);
 		g_signal_connect (menuitem, "drag_data_get",
@@ -1749,6 +1762,7 @@ main_menu_append (GtkWidget   *main_menu,
 	GtkWidget *item;
 	gboolean   add_separator;
 	GList     *children;
+	GList     *last;
 
 	if (!g_object_get_data (G_OBJECT (main_menu),
 				"panel-menu-needs-appending"))
@@ -1757,14 +1771,13 @@ main_menu_append (GtkWidget   *main_menu,
 	g_object_set_data (G_OBJECT (main_menu),
 			   "panel-menu-needs-appending", NULL);
 
-	children = gtk_container_get_children (GTK_CONTAINER (main_menu));
-
 	add_separator = FALSE;
-	if (children != NULL) {
-		while (children->next != NULL)
-			children = children->next;
-		add_separator = !GTK_IS_SEPARATOR (GTK_WIDGET (children->data));
+	children = gtk_container_get_children (GTK_CONTAINER (main_menu));
+	last = g_list_last (children);
+	if (last != NULL) {
+		add_separator = !GTK_IS_SEPARATOR (GTK_WIDGET (last->data));
 	}
+	g_list_free (children);
 
 	if (add_separator)
 		add_menu_separator (main_menu);
