@@ -40,6 +40,7 @@
 #include "panel-background.h"
 #include "panel-lockdown.h"
 #include "panel-stock-icons.h"
+#include "xstuff.h"
 
 #define PANEL_APPLET_FRAME_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), PANEL_TYPE_APPLET_FRAME, PanelAppletFramePrivate))
 
@@ -157,6 +158,12 @@ panel_applet_frame_sync_menu_state (PanelAppletFrame *frame)
 				      locked_down ? "1" : "0",
 				      NULL);
 
+	bonobo_ui_component_set_prop (frame->priv->ui_component,
+				      "/commands/LockSeparator",
+				      "hidden",
+				      locked_down ? "1" : "0",
+				      NULL);
+
 	bonobo_ui_component_set_prop (frame->priv->ui_component,
 				      "/commands/RemoveAppletFromPanel",
 				      "hidden",
@@ -363,9 +370,14 @@ panel_applet_frame_load_from_gconf (PanelWidget *panel_widget,
 	g_return_if_fail (id != NULL);
 
 	key = panel_gconf_full_key (PANEL_GCONF_APPLETS, id, "bonobo_iid");
-	applet_iid = gconf_client_get_string (panel_gconf_get_client (), key, NULL);
+	applet_iid = gconf_client_get_string (panel_gconf_get_client (),
+					      key, NULL);
 
-	panel_applet_frame_load (applet_iid, panel_widget, locked, position, TRUE, id);
+	if (!applet_iid)
+		return;
+
+	panel_applet_frame_load (applet_iid, panel_widget,
+				 locked, position, TRUE, id);
 
 	g_free (applet_iid);
 }
@@ -491,6 +503,14 @@ panel_applet_frame_change_background (PanelAppletFrame    *frame,
 	g_return_if_fail (PANEL_IS_APPLET_FRAME (frame));
 	g_return_if_fail (PANEL_IS_WIDGET (GTK_WIDGET (frame)->parent));
 
+	if (frame->priv->has_handle) {
+		PanelBackground *background;
+
+		background = &PANEL_WIDGET (GTK_WIDGET (frame)->parent)->background;
+		panel_background_change_background_on_widget (background,
+							      GTK_WIDGET (frame));
+	}
+
 	bg_str = panel_applet_frame_get_background_string (
 			frame, PANEL_WIDGET (GTK_WIDGET (frame)->parent), type);
 
@@ -511,6 +531,15 @@ panel_applet_frame_finalize (GObject *object)
 	panel_lockdown_notify_remove (G_CALLBACK (panel_applet_frame_sync_menu_state),
 				      frame);
 
+	if (frame->priv->control) {
+		/* do this before unref'ing every bonobo stuff since it looks
+		 * like we can receive some events when unref'ing them */
+		ORBit_small_unlisten_for_broken (frame->priv->control,
+						 G_CALLBACK (panel_applet_frame_cnx_broken));
+		CORBA_Object_release (frame->priv->control, NULL);
+		frame->priv->control = CORBA_OBJECT_NIL;
+	}
+
 	if (frame->priv->property_bag)
 		bonobo_object_release_unref (
 			frame->priv->property_bag, NULL);
@@ -523,13 +552,6 @@ panel_applet_frame_finalize (GObject *object)
 		bonobo_object_unref (
 			BONOBO_OBJECT (frame->priv->ui_component));
 
-	if (frame->priv->control) {
-		ORBit_small_unlisten_for_broken (frame->priv->control,
-						 G_CALLBACK (panel_applet_frame_cnx_broken));
-		CORBA_Object_release (frame->priv->control, NULL);
-		frame->priv->control = CORBA_OBJECT_NIL;
-	}
-
 	g_free (frame->priv->iid);
 	frame->priv->iid = NULL;
 
@@ -553,11 +575,11 @@ panel_applet_frame_paint (GtkWidget    *widget,
 		switch (frame->priv->orientation) {
 		case PANEL_ORIENTATION_TOP:
 		case PANEL_ORIENTATION_BOTTOM:
-			orientation = GTK_ORIENTATION_HORIZONTAL;
+			orientation = GTK_ORIENTATION_VERTICAL;
 			break;
 		case PANEL_ORIENTATION_LEFT:
 		case PANEL_ORIENTATION_RIGHT:
-			orientation = GTK_ORIENTATION_VERTICAL;
+			orientation = GTK_ORIENTATION_HORIZONTAL;
 			break;
 		default:
 			g_assert_not_reached ();
@@ -926,6 +948,9 @@ panel_applet_frame_cnx_broken (ORBitConnection  *cnx,
 
 	screen = gtk_widget_get_screen (GTK_WIDGET (frame));
 
+	if (xstuff_is_display_dead ())
+		return;
+
 	if (frame->priv->iid)
 		applet_name = panel_applet_frame_get_name (frame->priv->iid);
 
@@ -1028,7 +1053,9 @@ panel_applet_frame_loading_failed (PanelAppletFrame  *frame,
 	gtk_widget_show_all (dialog);
 
 	g_free (frame->priv->iid);
-	g_free (frame);
+	frame->priv->iid = NULL;
+
+	gtk_widget_destroy (GTK_WIDGET (frame));
 }
 
 static void
@@ -1264,7 +1291,7 @@ panel_applet_frame_activated (Bonobo_Unknown     object,
 	control_frame = bonobo_widget_get_control_frame (BONOBO_WIDGET (widget));
 	if (control_frame == NULL) {
 		g_warning (G_STRLOC ": failed to load applet %s "
-			   "(can't get control frame)", frame->priv->iid);
+			   "(cannot get control frame)", frame->priv->iid);
 		panel_applet_frame_loading_failed (frame, frame_act->id);
 		gtk_object_sink (GTK_OBJECT (widget));
 		g_free (frame_act->id);
@@ -1279,7 +1306,7 @@ panel_applet_frame_activated (Bonobo_Unknown     object,
 		error = bonobo_exception_get_text (&corba_ev);
 		CORBA_exception_free (&corba_ev);
 		g_warning (G_STRLOC ": failed to load applet %s "
-			   "(can't get property bag):\n%s",
+			   "(cannot get property bag):\n%s",
 			   frame->priv->iid, error);
 		panel_applet_frame_loading_failed (frame, frame_act->id);
 		gtk_object_sink (GTK_OBJECT (widget));
@@ -1302,7 +1329,7 @@ panel_applet_frame_activated (Bonobo_Unknown     object,
 		error = bonobo_exception_get_text (&corba_ev);
 		CORBA_exception_free (&corba_ev);
 		g_warning (G_STRLOC ": failed to load applet %s "
-			   "(can't get popup component):\n%s",
+			   "(cannot get popup component):\n%s",
 			   frame->priv->iid, error);
 		panel_applet_frame_loading_failed (frame, frame_act->id);
 		gtk_object_sink (GTK_OBJECT (widget));
@@ -1327,7 +1354,7 @@ panel_applet_frame_activated (Bonobo_Unknown     object,
 	if (!control) {
 		CORBA_exception_free (&corba_ev);
 		g_warning (G_STRLOC ": failed to load applet %s "
-			   "(can't get control)", frame->priv->iid);
+			   "(cannot get control)", frame->priv->iid);
 		panel_applet_frame_loading_failed (frame, frame_act->id);
 		gtk_object_sink (GTK_OBJECT (widget));
 		g_free (frame_act->id);
@@ -1339,7 +1366,7 @@ panel_applet_frame_activated (Bonobo_Unknown     object,
 	if (frame->priv->applet_shell == CORBA_OBJECT_NIL) {
 		CORBA_exception_free (&corba_ev);
 		g_warning (G_STRLOC ": failed to load applet %s "
-			   "(can't get applet shell)", frame->priv->iid);
+			   "(cannot get applet shell)", frame->priv->iid);
 		panel_applet_frame_loading_failed (frame, frame_act->id);
 		gtk_object_sink (GTK_OBJECT (widget));
 		g_free (frame_act->id);
