@@ -1,5 +1,5 @@
 /*
- * panel-menu-bar.h: panel Applications/Actions menu bar
+ * panel-menu-bar.h: panel Applications/Places/Desktop menu bar
  *
  * Copyright (C) 2003 Sun Microsystems, Inc.
  *
@@ -30,6 +30,8 @@
 
 G_BEGIN_DECLS
 
+#define PANEL_GNOME_LOGO_ICON "start-here"
+
 #define PANEL_TYPE_MENU_BAR         (panel_menu_bar_get_type ())
 #define PANEL_MENU_BAR(o)           (G_TYPE_CHECK_INSTANCE_CAST ((o), PANEL_TYPE_MENU_BAR, PanelMenuBar))
 #define PANEL_MENU_BAR_CLASS(k)     (G_TYPE_CHECK_CLASS_CAST((k), PANEL_TYPE_MENU_BAR, PanelMenuBarClass))
@@ -70,6 +72,10 @@ void       panel_menu_bar_popup_menu       (PanelMenuBar *menubar,
 
 void       panel_menu_bar_change_background (PanelMenuBar *menubar);
 
+void             panel_menu_bar_set_orientation (PanelMenuBar     *menubar,
+						 PanelOrientation  orientation);
+PanelOrientation panel_menu_bar_get_orientation (PanelMenuBar     *menubar);
+
 G_END_DECLS
 
 #endif /* __PANEL_MENU_BAR_H__ */
