@@ -410,6 +410,16 @@ panel_place_menu_item_append_gtk_bookmarks (GtkWidget *menu)
 				char *buffer;
 
 				buffer = gnome_vfs_get_local_path_from_uri (full_uri);
+				if (!buffer) {
+					g_free (tooltip);
+					g_free (full_uri);
+					if (bookmark->label)
+						g_free (bookmark->label);
+					gnome_vfs_uri_unref (bookmark->uri);
+					g_free (bookmark);
+					continue;
+				}
+
 				label = g_filename_display_basename (buffer);
 				g_free (buffer);
 			} else if (gnome_vfs_uri_is_local (bookmark->uri)) {
@@ -1045,6 +1055,7 @@ panel_menu_items_append_lock_logout (GtkWidget *menu)
 {
 	gboolean    separator_inserted;
 	GList      *children;
+	GList      *last;
 	GtkWidget  *item;
 	const char *translate;
 	char       *label;
@@ -1052,12 +1063,11 @@ panel_menu_items_append_lock_logout (GtkWidget *menu)
 
 	separator_inserted = FALSE;
 	children = gtk_container_get_children (GTK_CONTAINER (menu));
-
-	if (children != NULL) {
-		while (children->next != NULL)
-			children = children->next;
-		separator_inserted = GTK_IS_SEPARATOR (GTK_WIDGET (children->data));
+	last = g_list_last (children);
+	if (last != NULL) {
+		separator_inserted = GTK_IS_SEPARATOR (GTK_WIDGET (last->data));
 	}
+	g_list_free (children);
 
 	if (panel_lock_screen_action_available ("lock")) {
 		item = panel_menu_items_create_action_item (PANEL_ACTION_LOCK);
