@@ -183,6 +183,10 @@ recent_documents_clear_cb (GtkMenuItem      *menuitem,
 	g_signal_connect (clear_recent_dialog, "response",
 			  G_CALLBACK (clear_dialog_response), manager);
 
+	g_signal_connect (clear_recent_dialog, "destroy",
+			  G_CALLBACK (gtk_widget_destroyed),
+			  &clear_recent_dialog);
+
 	tmp = &clear_recent_dialog;
 	g_object_add_weak_pointer (G_OBJECT (clear_recent_dialog), tmp);
 
@@ -233,6 +237,7 @@ panel_recent_append_documents_menu (GtkWidget        *top_menu,
 				 G_CALLBACK (panel_recent_manager_changed_cb),
 				 menu_item, 0);
 
+	g_object_get (manager, "size", &size, NULL);
 	gtk_widget_set_sensitive (menu_item, size > 0);
 
 	add_menu_separator (recent_menu);
