@@ -40,7 +40,6 @@
 #include <glib/gi18n.h>
 #include <gdk/gdkkeysyms.h>
 #include <glade/glade-xml.h>
-#include <libgnome/gnome-desktop-item.h>
 #include <libgnome/gnome-exec.h>
 #include <libgnome/gnome-util.h>
 #include <libgnomeui/gnome-entry.h>
@@ -96,6 +95,7 @@ typedef struct {
 	gboolean	  completion_started;
 	
 	char		 *icon_path;
+	char             *desktop_path;
 	char		 *item_name;	
 } PanelRunDialog;
 
@@ -126,7 +126,11 @@ panel_run_dialog_destroy (PanelRunDialog *dialog)
 	dialog->add_icon_paths = NULL;
 
 	g_free (dialog->icon_path);
+	dialog->icon_path = NULL;
+	g_free (dialog->desktop_path);
+	dialog->desktop_path = NULL;
 	g_free (dialog->item_name);
+	dialog->item_name = NULL;
 
 	if (dialog->add_icons_idle_id)
 		g_source_remove (dialog->add_icons_idle_id);
@@ -295,11 +299,14 @@ panel_run_dialog_launch_command (PanelRunDialog *dialog,
 				      &error);
 			
 	if (!result) {
-		panel_error_dialog (screen, "cannot_spawn_command", TRUE,
-				    _("Could not run command '%s'"),
-				    "%s",
-				    escaped,
-				    error->message);
+		char *primary;
+
+		primary = g_strdup_printf (_("Could not run command '%s'"),
+					   escaped);
+		panel_error_dialog (GTK_WINDOW (dialog->run_dialog), NULL,
+				    "cannot_spawn_command", TRUE,
+				    primary, error->message);
+		g_free (primary);
 
 		g_error_free (error);
 	}
@@ -321,11 +328,14 @@ panel_run_dialog_show_url (PanelRunDialog *dialog,
 
 	gnome_url_show_on_screen (url, screen, &error);
 	if (error) {
-		panel_error_dialog (screen, "cannot_show_url", TRUE,
-				    _("Could not open location '%s'"),
-				    "%s",
-				    escaped,
-				    error->message);
+		char *primary;
+
+		primary = g_strdup_printf (_("Could not open location '%s'"),
+					   escaped);
+		panel_error_dialog (GTK_WINDOW (dialog->run_dialog), NULL,
+				    "cannot_show_url", TRUE,
+				    primary, error->message);
+		g_free (primary);
 
 		g_error_free (error);
 		return FALSE;
@@ -373,11 +383,14 @@ panel_run_dialog_execute (PanelRunDialog *dialog)
 	disk = g_locale_from_utf8 (command, -1, NULL, NULL, &error);
 
 	if (!disk || error) {
-		panel_error_dialog (screen, "cannot_convert_command_from_utf8",
-				    TRUE, _("Could not convert '%s' from UTF-8"),
-				    "%s",
-				    command,
-				    error->message);
+		char *primary;
+
+		primary = g_strdup_printf (_("Could not convert '%s' from UTF-8"),
+					   command);
+		panel_error_dialog (GTK_WINDOW (dialog->run_dialog), NULL,
+				    "cannot_convert_command_from_utf8", TRUE,
+				    primary, error->message);
+		g_free (primary);
 
 		g_error_free (error);
 		return;
@@ -654,7 +667,6 @@ panel_run_dialog_find_command_idle (PanelRunDialog *dialog)
 	
 	g_free (dialog->item_name);
 	dialog->item_name = found_name;
-	//FIXME err... why is this useful??? don't we want the .desktop path for dnd?
 	
 	dialog->find_command_idle_id = 0;
 	return FALSE;
@@ -950,14 +962,13 @@ static void
 program_list_selection_changed (GtkTreeSelection *selection,
 				PanelRunDialog   *dialog)
 {
-	GnomeDesktopItem *ditem;
-	GtkTreeModel     *filter_model;
-	GtkTreeModel     *child_model;
-	GtkTreeIter       iter;
-	GtkTreeIter       filter_iter;
-	const char       *temp;
-	char             *path, *stripped;
-	gboolean          terminal;
+	GtkTreeModel *filter_model;
+	GtkTreeModel *child_model;
+	GtkTreeIter   iter;
+	GtkTreeIter   filter_iter;
+	char         *temp;
+	char         *path, *stripped;
+	gboolean      terminal;
 		
 	if (!gtk_tree_selection_get_selected (selection, &filter_model,
 					      &filter_iter))
@@ -973,46 +984,60 @@ program_list_selection_changed (GtkTreeSelection *selection,
 			    -1);
 				  
 	if (path) {
-		ditem = gnome_desktop_item_new_from_file (path,
-							  GNOME_DESKTOP_ITEM_LOAD_ONLY_IF_EXISTS,
-							  NULL /* error */);
-		if (ditem) {
+		GKeyFile *key_file;
+
+		key_file = g_key_file_new ();
+
+		if (g_key_file_load_from_file (key_file, path,
+					       G_KEY_FILE_NONE, NULL)) {
 			dialog->use_program_list = TRUE;
+			if (dialog->desktop_path)
+				g_free (dialog->desktop_path);
+			dialog->desktop_path = g_strdup (path);
+			if (dialog->item_name)
+				g_free (dialog->item_name);
+			dialog->item_name = NULL;
 			
+			gtk_tree_model_foreach (GTK_TREE_MODEL (dialog->program_list_store),
+						panel_run_dialog_make_all_list_visible,
+						NULL);
+
 			/* Order is important here. We have to set the text
 			 * first so that the drag source is enabled, otherwise
 			 * the drag icon can't be set by
 			 * panel_run_dialog_set_icon.
 			 */
-			temp = gnome_desktop_item_get_string (ditem, GNOME_DESKTOP_ITEM_EXEC);
+			temp = panel_util_key_file_get_string (key_file,
+							       "Exec");
 			if (temp) {
 				stripped = remove_parameters (temp);
 				gtk_entry_set_text (GTK_ENTRY (dialog->gtk_entry), stripped);
 				g_free (stripped);
 			} else {
-				temp = gnome_desktop_item_get_string (ditem, GNOME_DESKTOP_ITEM_URL);
+				temp = panel_util_key_file_get_string (key_file,
+								       "URL");
 				gtk_entry_set_text (GTK_ENTRY (dialog->gtk_entry), sure_string (temp));
 			}
+			g_free (temp);
 
-			temp = gnome_desktop_item_get_string (ditem, GNOME_DESKTOP_ITEM_ICON);
+			temp = panel_util_key_file_get_locale_string (key_file,
+								      "Icon");
 			panel_run_dialog_set_icon (dialog, temp);
+			g_free (temp);
 			
-			temp = gnome_desktop_item_get_localestring (ditem, GNOME_DESKTOP_ITEM_COMMENT);
+			temp = panel_util_key_file_get_locale_string (key_file,
+								      "Comment");
 			//FIXME: if sure_string () == "", we should display "Will run..." as in entry_changed()
 			gtk_label_set_text (GTK_LABEL (dialog->program_label), sure_string (temp));
-			
-			terminal = gnome_desktop_item_get_boolean (ditem, GNOME_DESKTOP_ITEM_TERMINAL);
+			g_free (temp);
+
+			terminal = panel_util_key_file_get_boolean (key_file,
+								    "Terminal",
+								    FALSE);
 			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->terminal_checkbox),
 						      terminal);
 
-			if (dialog->item_name)
-				g_free (dialog->item_name);
-
-			dialog->item_name = g_strdup (gnome_desktop_item_get_string (
-							      ditem,
-							      GNOME_DESKTOP_ITEM_NAME));
-
-			gnome_desktop_item_unref (ditem);
+			g_key_file_free (key_file);
                 }
 
 		g_free (path);
@@ -1404,17 +1429,20 @@ entry_event (GtkEditable    *entry,
 	     GdkEventKey    *event,
 	     PanelRunDialog *dialog)
 {
-	char *prefix;
-	char *nospace_prefix;
-	char *nprefix;
-	char *temp;
-	int   pos, tmp;
+	GtkTreeSelection *selection;
+	char             *prefix;
+	char             *nospace_prefix;
+	char             *nprefix;
+	char             *temp;
+	int               pos, tmp;
 
 	if (event->type != GDK_KEY_PRESS)
 		return FALSE;
 
 	/* if user typed something we're not using the list anymore */
 	dialog->use_program_list = FALSE;
+	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (dialog->program_list));
+	gtk_tree_selection_unselect_all (selection);
 
 	if (!panel_profile_get_enable_autocompletion ())
 		return FALSE;
@@ -1523,9 +1551,15 @@ entry_changed (GtkEntry       *entry,
 		start++;
 
 	/* update item name to use for dnd */
-	if (!dialog->use_program_list && dialog->item_name) {
-		g_free (dialog->item_name);
-		dialog->item_name = NULL;
+	if (!dialog->use_program_list) {
+		if (dialog->desktop_path) {
+			g_free (dialog->desktop_path);
+			dialog->desktop_path = NULL;
+		}
+		if (dialog->item_name) {
+			g_free (dialog->item_name);
+			dialog->item_name = NULL;
+		}
 	}
 
 	/* desensitize run button if no text entered */
@@ -1533,8 +1567,9 @@ entry_changed (GtkEntry       *entry,
 		gtk_widget_set_sensitive (dialog->run_button, FALSE);
 		gtk_drag_source_unset (dialog->run_dialog);
 
-		gtk_label_set_text (GTK_LABEL (dialog->program_label),
-				    _("Select an application to view its description."));
+		if (panel_profile_get_enable_program_list ())
+			gtk_label_set_text (GTK_LABEL (dialog->program_label),
+					    _("Select an application to view its description."));
 
 		panel_run_dialog_set_default_icon (dialog, FALSE);
 
@@ -1570,7 +1605,8 @@ entry_changed (GtkEntry       *entry,
 			     GDK_ACTION_COPY);
 	gtk_drag_source_add_uri_targets (dialog->run_dialog);
 
-	if (!dialog->use_program_list) {
+	if (panel_profile_get_enable_program_list () &&
+	    !dialog->use_program_list) {
 		msg = g_strdup_printf (_("Will run command: '%s'"),
 				       start);
 		gtk_label_set_text (GTK_LABEL (dialog->program_label), msg);
@@ -1670,7 +1706,73 @@ panel_run_dialog_setup_entry (PanelRunDialog *dialog,
 			  G_CALLBACK (entry_drag_data_received), dialog);
 }
 
-static void  
+static char *
+panel_run_dialog_create_desktop_file (PanelRunDialog *dialog)
+{
+	GKeyFile   *key_file;
+	gboolean    exec = FALSE;
+	const char *text;
+	char       *uri;
+	char       *disk;
+	char       *scheme;
+
+	text = gtk_entry_get_text (GTK_ENTRY (dialog->gtk_entry));
+	
+	if (!text || !text [0])
+		return NULL;
+		
+	key_file = panel_util_key_file_new_desktop ();
+
+	disk = g_locale_from_utf8 (text, -1, NULL, NULL, NULL);
+	uri = gnome_vfs_make_uri_from_input_with_dirs (disk,
+						       GNOME_VFS_MAKE_URI_DIR_HOMEDIR);
+	g_free (disk);
+
+	scheme = gnome_vfs_get_uri_scheme (uri);
+	if (!g_ascii_strcasecmp (scheme, "http"))
+		exec = command_is_executable (text);
+	else if (!g_ascii_strcasecmp (scheme, "file"))
+		exec = command_is_executable (text);
+	g_free (scheme);
+		
+	if (exec) {
+		panel_util_key_file_set_string (key_file,
+						"Type", "Application");
+		panel_util_key_file_set_string (key_file,
+						"Exec", text);
+	} else {
+		panel_util_key_file_set_string (key_file,
+						"Type", "Link");
+		panel_util_key_file_set_string (key_file,
+						"URL", uri);
+	}
+	g_free (uri);
+
+	panel_util_key_file_set_locale_string (key_file, "Name",
+					       (dialog->item_name) ?
+					       	dialog->item_name : text);
+
+	panel_util_key_file_set_boolean (key_file, "Terminal",
+					 gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->terminal_checkbox)));
+
+	panel_util_key_file_set_locale_string (key_file, "Icon",
+					       dialog->icon_path);
+
+	uri = panel_make_unique_uri (g_get_tmp_dir (), ".desktop");
+	disk = g_filename_from_uri (uri, NULL, NULL);
+
+	if (!disk || !panel_util_key_file_to_file (key_file, disk, NULL)) {
+		g_free (uri);
+		uri = NULL;
+	}
+
+	g_key_file_free (key_file);
+	g_free (disk);
+
+	return uri;
+}
+
+static void
 pixmap_drag_data_get (GtkWidget          *run_dialog,
 	  	      GdkDragContext     *context,
 		      GtkSelectionData   *selection_data,
@@ -1678,66 +1780,19 @@ pixmap_drag_data_get (GtkWidget          *run_dialog,
 		      guint               time,
 		      PanelRunDialog     *dialog)
 {
-	GnomeDesktopItem *ditem;
-	gboolean          exec = FALSE;
-	const char       *text;
-	char             *uri;
-	char             *disk;
-	char             *scheme;
-	
-	text = gtk_entry_get_text (GTK_ENTRY (dialog->gtk_entry));
-	
-	if (!text || !text [0])
-		return;
-		
-	ditem = gnome_desktop_item_new ();
+	char *uri;
 
-	disk = g_locale_from_utf8 (text, -1, NULL, NULL, NULL);
-	uri = gnome_vfs_make_uri_from_input_with_dirs (disk,
-						       GNOME_VFS_MAKE_URI_DIR_HOMEDIR);
-	scheme = gnome_vfs_get_uri_scheme (uri);
-	
-	if (!g_ascii_strcasecmp (scheme, "http"))
-		exec = command_is_executable (text);
-		
-	else if (!g_ascii_strcasecmp (scheme, "file"))
-		exec = command_is_executable (uri);
-		
-	if (exec) {
-		gnome_desktop_item_set_entry_type (ditem, GNOME_DESKTOP_ITEM_TYPE_APPLICATION);
-		gnome_desktop_item_set_string (ditem, GNOME_DESKTOP_ITEM_EXEC, text);
-	} else {
-		gnome_desktop_item_set_entry_type (ditem, GNOME_DESKTOP_ITEM_TYPE_LINK);
-		gnome_desktop_item_set_string (ditem, GNOME_DESKTOP_ITEM_URL, uri);
-	}
-		
-	gnome_desktop_item_set_string (ditem,
-				       GNOME_DESKTOP_ITEM_NAME, (dialog->item_name) ?
-				              dialog->item_name : text);
+	if (dialog->use_program_list && dialog->desktop_path)
+		uri = g_strdup (dialog->desktop_path);
+	else
+		uri = panel_run_dialog_create_desktop_file (dialog);
 
-	gnome_desktop_item_set_boolean (ditem,
-					GNOME_DESKTOP_ITEM_TERMINAL,
-					gtk_toggle_button_get_active (
-						 GTK_TOGGLE_BUTTON (dialog->terminal_checkbox)));
-
-	gnome_desktop_item_set_string (ditem,
-				       GNOME_DESKTOP_ITEM_ICON, 
-				       dialog->icon_path);
-	
-	g_free (uri);
-
-	uri = panel_make_unique_uri (g_get_tmp_dir (), ".desktop");
-	gnome_desktop_item_set_location (ditem, uri);
-
-	if (gnome_desktop_item_save (ditem, NULL, FALSE, NULL))
+	if (uri) {
 		gtk_selection_data_set (selection_data,
 					selection_data->target, 8,
 					(unsigned char *) uri, strlen (uri));
-	gnome_desktop_item_unref (ditem);
-
-	g_free (uri);
-	g_free (disk);
-	g_free (scheme);
+		g_free (uri);
+	}
 }
 
 static void
@@ -1825,10 +1880,15 @@ panel_run_dialog_present (GdkScreen *screen,
 			     "panel_run_dialog",
 			     NULL);
 	if (!gui) {
-		panel_error_dialog (screen, "cannot_display_run_dialog", TRUE,
+		char *secondary;
+
+		secondary = g_strdup_printf (_("Unable to load file '%s'."),
+					     GLADEDIR"/panel-run-dialog.glade");
+		panel_error_dialog (NULL, screen, "cannot_display_run_dialog",
+				    TRUE,
 				    _("Could not display run dialog"),
-				    _("Unable to load file '%s'."),
-				      GLADEDIR "/panel-run-dialog.glade");
+				      secondary);
+		g_free (secondary);
 		return;
 	}
 
